@@ -10,25 +10,9 @@
 (*  library, see the COPYING file for more information.                               *)
 (**************************************************************************************)
 
-(** Dependency solver. Low Level API *)
-
-(** Implementation of the EDOS algorithms (and more).
-    This module respects the cudf semantic. 
-
-    This module contains two type of functions.
-    Normal functions work on a cudf universe. These are just a wrapper to
-    _cache functions.
-
-    _cache functions work on a pool of ids that is a more compact
-    representation of a cudf universe based on arrays of integers.
-    _cache function can be used to avoid recreating the pool for each
-    operation and therefore speed up operations.
-*)
-
 open ExtLib
 open Common
 
-(** progress bar *)
 let progressbar_init = Util.Progress.create "Depsolver_int.init_solver"
 let progressbar_univcheck = Util.Progress.create "Depsolver_int.univcheck"
 
@@ -39,17 +23,19 @@ include Util.Logging(struct let label = label end) ;;
 module R = struct type reason = Diagnostic.reason_int end
 module S = EdosSolver.M(R)
 
-(** low level solver data type *)
 type solver = {
-  constraints : S.state; (** the sat problem *)
-  map : Util.projection (** a map from cudf package ids to solver ids *)
+  constraints : S.state;
+  map : Util.projection;
+  globalid : (bool * bool) * int
 }
 
+type global_constraints = (Cudf_types.vpkglist * int list) list
+
 type dep_t = 
   ((Cudf_types.vpkg list * S.var list) list * 
    (Cudf_types.vpkg * S.var list) list ) 
 and pool = dep_t array
-and t = [`SolverPool of pool | `CudfPool of pool]
+and t = [`SolverPool of pool | `CudfPool of (bool * pool)]
 
 type result =
   |Success of (unit -> int list)
@@ -58,18 +44,18 @@ type result =
 (* cudf uid -> cudf uid array . Here we assume cudf uid are sequential
    and we can use them as an array index *)
 let init_pool_univ ~global_constraints univ =
-  let size = (Cudf.universe_size univ) + 1 in
   (* the last element of the array *)
-  let globalid = size - 1 in
+  let size = Cudf.universe_size univ in
   let keep = Hashtbl.create 200 in
   let pool = 
     (* here I initalize the pool to size + 1, that is I reserve one spot
      * to encode the global constraints associated with the universe.
      * However, since they are global, I've to add the at the end, after
      * I have analyzed all packages in the universe. *)
-    Array.init size (fun uid ->
+    Array.init (size + 1) (fun uid ->
       try
-        if uid = globalid then ([],[]) else begin (* the last index *)
+        if uid = size then ([],[])  (* the last index *)
+        else begin
           let pkg = Cudf.package_by_uid univ uid in
           let dll = 
             List.map (fun vpkgs ->
@@ -77,38 +63,51 @@ let init_pool_univ ~global_constraints univ =
             ) pkg.Cudf.depends 
           in
           let cl = 
-            List.map (fun vpkg ->
-              debug "Conflict %s" (Cudf_types_pp.string_of_vpkg vpkg);
-              (vpkg, CudfAdd.resolve_vpkg_int univ vpkg)
+            List.filter_map (fun vpkg ->
+              match CudfAdd.resolve_vpkg_int univ vpkg with
+              |[] -> None
+              |l -> Some (vpkg, l)
             ) pkg.Cudf.conflicts
           in
-          if pkg.Cudf.keep = `Keep_package then
-            CudfAdd.add_to_package_list keep (pkg.Cudf.package,None) uid;
-            (*
-          if pkg.Cudf.keep = `Keep_version then
-            CudfAdd.add_to_package_list keep (pkg.Cudf.package,Some (`Eq pkg.Cudf.version)) uid;
-            *)
+          if pkg.Cudf.installed then begin
+            match pkg.Cudf.keep with
+            |`Keep_none -> ()
+            |`Keep_package ->
+              List.iter (fun id ->
+                CudfAdd.add_to_package_list keep (pkg.Cudf.package,None) id
+              ) (CudfAdd.resolve_vpkg_int univ (pkg.Cudf.package,None));
+            |`Keep_version ->
+              CudfAdd.add_to_package_list keep 
+                (pkg.Cudf.package,Some (`Eq, pkg.Cudf.version)) uid;
+            |`Keep_feature ->
+              List.iter (function
+                |(name,None) ->
+                  List.iter (fun id ->
+                    CudfAdd.add_to_package_list keep (name,None) id
+                  ) (CudfAdd.resolve_vpkg_int univ (name,None));
+                |(name,Some(`Eq,v)) ->
+                  List.iter (fun id ->
+                    CudfAdd.add_to_package_list keep (name,Some(`Eq,v)) id
+                  ) (CudfAdd.resolve_vpkg_int univ (name,Some(`Eq,v)));
+              ) pkg.Cudf.provides
+          end ;
           (dll,cl)
         end
       with Not_found ->
-        fatal "Package uid not found during solver pool initialization. Packages uid must have no gaps in the given universe"
+        fatal "Package uid (%d) not found during solver pool initialization. Packages uid must have no gaps in the given universe" uid
     )
   in
-  if global_constraints then begin
-    let keep_dll =
-      Hashtbl.fold (fun cnstr {contents = l} acc ->
-        ([cnstr],l) :: acc
-      ) keep []
-    in
-    (* here in theory we could encode more complex contraints .
-     * for the moment we consider only `Keep_package *)
-    pool.(globalid) <- (keep_dll,[])
-  end;
-  (`CudfPool pool)
+  let keep_dll =
+    Hashtbl.fold (fun cnstr {contents = l} acc ->
+      ([cnstr],l) :: acc
+    ) keep global_constraints
+  in
+  pool.(size) <- (keep_dll,[]);
+  (`CudfPool (keep_dll <> [],pool))
 
 (** this function creates an array indexed by solver ids that can be 
     used to init the edos solver *)
-let init_solver_pool map (`CudfPool cudfpool) closure =
+let init_solver_pool map (`CudfPool (keep_constraints,cudfpool)) closure =
   let convert (dll,cl) =
     let sdll = 
       List.map (fun (vpkgs,uidl) ->
@@ -135,41 +134,30 @@ let init_solver_pool map (`CudfPool cudfpool) closure =
     in (sdll,scl)
   in
 
-  (* while in init_pool_univ we create a pool that is bigger
-   * then the universe to keep into consideration the space
-   * needed to encode the global contraint, here we assume that
-   * the pool is already of the correct size and the closure
-   * contains also the globalid *) 
-  let globalid = Array.length cudfpool in
   let solverpool = 
-    let size = List.length closure in
-    Array.init size (fun sid ->
-      if sid = globalid then 
-        convert cudfpool.(globalid)
-      else
-        let uid = map#inttovar sid in
-        let (dll,cl) = cudfpool.(uid) in
-        convert (dll,cl)
+    Array.init (List.length closure) (fun sid ->
+      convert (cudfpool.(map#inttovar sid))
     )
   in
   (`SolverPool solverpool)
 
 (** initalise the sat solver. operate only on solver ids *)
-let init_solver_cache ?(buffer=false) (`SolverPool varpool) =
+let init_solver_cache ?(buffer=false) ?(explain=true) (`SolverPool varpool) =
   let num_conflicts = ref 0 in
   let num_disjunctions = ref 0 in
   let num_dependencies = ref 0 in
+  let if_explain l = if explain then l else [] in
   let varsize = Array.length varpool in
 
   let add_depend constraints vpkgs pkg_id l =
     let lit = S.lit_of_var pkg_id false in 
     if (List.length l) = 0 then 
-      S.add_rule constraints [|lit|] [Diagnostic.MissingInt(pkg_id,vpkgs)]
+      S.add_rule constraints [|lit|] (if_explain [Diagnostic.MissingInt(pkg_id,vpkgs)])
     else begin
       let lits = List.map (fun id -> S.lit_of_var id true) l in
       num_disjunctions := !num_disjunctions + (List.length lits);
       S.add_rule constraints (Array.of_list (lit :: lits))
-        [Diagnostic.DependencyInt (pkg_id, vpkgs, l)];
+        (if_explain [Diagnostic.DependencyInt (pkg_id, vpkgs, l)]);
       if (List.length lits) > 1 then
         S.associate_vars constraints (S.lit_of_var pkg_id true) l;
     end
@@ -185,7 +173,7 @@ let init_solver_cache ?(buffer=false) (`SolverPool varpool) =
         Util.IntPairHashtbl.add conflicts pair ();
         let p = S.lit_of_var i false in
         let q = S.lit_of_var j false in
-        S.add_rule constraints [|p;q|] [Diagnostic.ConflictInt(i,j,vpkg)];
+        S.add_rule constraints [|p;q|] (if_explain [Diagnostic.ConflictInt(i,j,vpkg)]);
       end
     end
   in
@@ -213,7 +201,7 @@ let init_solver_cache ?(buffer=false) (`SolverPool varpool) =
     exec_depends constraints id dll;
     exec_conflicts constraints id cl
   ) varpool;
-    
+
   Util.IntPairHashtbl.clear conflicts;
 
   debug "n. disjunctions %d" !num_disjunctions;
@@ -227,46 +215,46 @@ let init_solver_cache ?(buffer=false) (`SolverPool varpool) =
   
     @param tested: optional int array used to cache older results
 *)
-let solve ?tested solver request =
+let solve ?tested ~explain solver request =
   S.reset solver.constraints;
 
   let result solve collect var =
     (* Real call to the SAT solver *)
     if solve solver.constraints var then begin
-      let l = S.assignment_true solver.constraints in
-      List.iter (fun i -> 
+      if explain then (
+        let l = S.assignment_true solver.constraints in
+        if not(Option.is_none tested) then
+          List.iter (fun i -> (Option.get tested).(i) <- true) l;
+        Diagnostic.SuccessInt(fun ?(all=false) () -> l)
+      ) else (
         if not(Option.is_none tested) then (
-          (Option.get tested).(i) <- true;
-        )
-      ) l;
-      let get_assignent ?(all=true) () = List.map solver.map#inttovar l in
-      Diagnostic.SuccessInt(get_assignent)
+          let l = S.assignment_true solver.constraints in
+          List.iter (fun i -> (Option.get tested).(i) <- true) l);
+        Diagnostic.SuccessInt(fun ?(all=false) () -> [])
+      )
     end else
-      let get_reasons () = collect solver.constraints var in
-      Diagnostic.FailureInt(get_reasons)
+      if explain then
+        Diagnostic.FailureInt(fun () -> collect solver.constraints var)
+      else
+        Diagnostic.FailureInt(fun () -> [])
   in
+  match request,solver.globalid with
+  |[],((false,false),_) -> Diagnostic.SuccessInt(fun ?(all=false) () -> [])
+  |[],(((_,true)|(true,_)),gid) -> result S.solve S.collect_reasons (solver.map#vartoint gid)
+  |[i],((false,false),_) -> result S.solve S.collect_reasons (solver.map#vartoint i)
+  |l,((false,false),_) ->
+      let il = List.map solver.map#vartoint l in
+      result S.solve_lst S.collect_reasons_lst il
+  |l,(_,gid) ->
+      let il = List.map solver.map#vartoint (gid :: l) in
+      result S.solve_lst S.collect_reasons_lst il
 
-  match request with
-  |(None,[i]) ->
-      result S.solve S.collect_reasons (solver.map#vartoint i)
-  |(None,il) ->
-      result S.solve_lst S.collect_reasons_lst (List.map solver.map#vartoint il)
-
-  |(Some k,[i]) ->
-      result S.solve_lst S.collect_reasons_lst (List.map solver.map#vartoint [k;i])
-  |(Some k,il) ->
-      result S.solve_lst S.collect_reasons_lst (List.map solver.map#vartoint (k::il))
-
-(* this function is used to "distcheck" a list of packages *)
-let pkgcheck global_constraints callback solver tested id =
-  (* global id is a fake package id encoding the global constraints of the
-   * universe. it is the last element of the id array *)
-  let globalid = (Array.length tested) - 1 in
-  let req = if global_constraints then begin (Some globalid,[id]) end else (None,[id]) in
+(* this function is used to "distcheck" a list of packages. The id is a cudfpool index *)
+let pkgcheck callback explain solver tested id =
   let res =
     Util.Progress.progress progressbar_univcheck;
     if not(tested.(id)) then
-      solve ~tested solver req 
+      solve ~tested ~explain solver [id]
     else begin
       (* this branch is true only if the package was previously
          added to the tested packages and therefore it is installable
@@ -274,35 +262,40 @@ let pkgcheck global_constraints callback solver tested id =
          of installed packages despite the fact the the package was already
          tested. This is done to provide one installation set for each package
          in the universe *)
-      let f ?(all=false) () =
-        if all then begin
-          match solve solver req with
-          |Diagnostic.SuccessInt(f_int) -> f_int ()
-          |Diagnostic.FailureInt _ -> assert false (* impossible *)
-        end else []
-      in Diagnostic.SuccessInt(f) 
+      if explain then
+        let f ?(all=false) () =
+          if all then begin
+            match solve solver ~explain [id] with
+            |Diagnostic.SuccessInt(f_int) -> f_int ()
+            |Diagnostic.FailureInt _ -> assert false (* impossible *)
+          end else []
+        in Diagnostic.SuccessInt(f)
+      (* avoid to allocate anything on the stack if not stricly needed *)
+      else Diagnostic.SuccessInt(fun ?(all=false) () -> [])
     end
   in
   match callback, res with
   |None, Diagnostic.SuccessInt _ -> true
   |None, Diagnostic.FailureInt _ -> false
-  |Some f, Diagnostic.SuccessInt _ -> ( f (res,req) ; true )
-  |Some f, Diagnostic.FailureInt _ -> ( f (res,req) ; false )
+  |Some f, Diagnostic.SuccessInt _ -> ( f (res,[id]) ; true )
+  |Some f, Diagnostic.FailureInt _ -> ( f (res,[id]) ; false )
 
 (** low level constraint solver initialization
  
     @param buffer debug buffer to print out debug messages
     @param univ cudf package universe
 *)
-let init_solver_univ ?(global_constraints=true) ?(buffer=false) univ =
+let init_solver_univ ~global_constraints ?(buffer=false) ?(explain=true) univ =
   let map = new Util.identity in
   (* here we convert a cudfpool in a varpool. The assumption
    * that cudf package identifiers are contiguous is essential ! *)
-  let `CudfPool pool = init_pool_univ global_constraints univ in
+  let `CudfPool (keep_constraints,pool) = init_pool_univ ~global_constraints univ in
   let varpool = `SolverPool pool in
-  let constraints = init_solver_cache ~buffer varpool in
-  let solver = { constraints = constraints ; map = map } in
-  solver
+  let constraints = init_solver_cache ~buffer ~explain varpool in
+  let gid = Cudf.universe_size univ in
+  let global_constraints = global_constraints <> [] in
+  { constraints = constraints ; map = map;
+    globalid = ((keep_constraints,global_constraints),gid) }
 
 (** low level constraint solver initialization
  
@@ -311,13 +304,19 @@ let init_solver_univ ?(global_constraints=true) ?(buffer=false) univ =
     @param closure subset of packages used to initialize the solver
 *)
 (* pool = cudf pool - closure = dependency clousure . cudf uid list *)
-let init_solver_closure ?(buffer=false) cudfpool closure =
+let init_solver_closure ~global_constraints ?(buffer=false) 
+  (`CudfPool (keep_constraints,cudfpool)) closure =
+  let gid = Array.length cudfpool - 1 in
+  let global_constraints = global_constraints <> [] in
   let map = new Util.intprojection (List.length closure) in
   List.iter map#add closure;
-  let varpool = init_solver_pool map cudfpool closure in
+  let varpool =
+    init_solver_pool map
+      (`CudfPool (keep_constraints,cudfpool)) closure
+  in
   let constraints = init_solver_cache ~buffer varpool in
-  let solver = { constraints = constraints ; map = map } in
-  solver
+  { constraints ; map = map;
+    globalid = ((keep_constraints,global_constraints),gid) }
 
 (** return a copy of the state of the solver *)
 let copy_solver solver =
@@ -336,7 +335,7 @@ let reverse_dependencies univ =
   Cudf.iteri_packages (fun i p ->
     List.iter (fun ll ->
       List.iter (fun q ->
-        let j = CudfAdd.vartoint univ q in
+        let j = CudfAdd.pkgtoint univ q in
         if i <> j then
           if not(List.mem i reverse.(j)) then
             reverse.(j) <- i::reverse.(j)
@@ -345,10 +344,12 @@ let reverse_dependencies univ =
   ) univ;
   reverse
 
-let dependency_closure_cache ?(maxdepth=max_int) ?(conjunctive=false) (`CudfPool cudfpool) idlist =
+let dependency_closure_cache ?(maxdepth=max_int) 
+  ?(conjunctive=false) (`CudfPool (_,cudfpool)) idlist =
   let queue = Queue.create () in
+  let globalid = (Array.length cudfpool - 1) in
   let visited = Hashtbl.create (2 * (List.length idlist)) in
-  List.iter (fun e -> Queue.add (e,0) queue) (CudfAdd.normalize_set idlist);
+  List.iter (fun e -> Queue.add (e,0) queue) (CudfAdd.normalize_set (globalid::idlist));
   while (Queue.length queue > 0) do
     let (id,level) = Queue.take queue in
     if not(Hashtbl.mem visited id) && level < maxdepth then begin
@@ -369,21 +370,6 @@ let dependency_closure_cache ?(maxdepth=max_int) ?(conjunctive=false) (`CudfPool
   done;
   Hashtbl.fold (fun k _ l -> k::l) visited []
 
-(** [dependency_closure index l] return the union of the dependency closure of
-    all packages in [l] .
-
-    @param maxdepth the maximum cone depth (infinite by default)
-    @param conjunctive consider only conjunctive dependencies (false by default)
-    @param universe the package universe
-    @param pkglist a subset of [universe]
-*)
-let dependency_closure ?(maxdepth=max_int) ?(conjunctive=false) ?(global_constraints=false) universe pkglist =
-  let pool = init_pool_univ ~global_constraints universe in
-  let globalid = Cudf.universe_size universe in
-  let idlist = List.map (CudfAdd.vartoint universe) pkglist in
-  let l = dependency_closure_cache ~maxdepth ~conjunctive pool (globalid::idlist) in
-  List.filter_map (fun p -> if p <> globalid then Some (CudfAdd.inttovar universe p) else None) l
-
 (*    XXX : elements in idlist should be included only if because
  *    of circular dependencies *)
 (** return the dependency closure of the reverse dependency graph.
