@@ -10,19 +10,11 @@
 (*  library, see the COPYING file for more information.                               *)
 (**************************************************************************************)
 
-(** {2 Low level Integer Un-installability reasons} *)
-type reason_int =
-  |DependencyInt of (int * Cudf_types.vpkg list * int list)
-  |MissingInt of (int * Cudf_types.vpkg list)
-  |ConflictInt of (int * int * Cudf_types.vpkg)
-
-type result_int =
-  |SuccessInt of (?all:bool -> unit -> int list)
-  |FailureInt of (unit -> reason_int list)
-
-type request_int = (int option * int list)
-
 (** {2 Un-installability reasons} *)
+
+(** The request provided to the solver *)
+type request = Cudf.package list
+
 type reason =
   |Dependency of (Cudf.package * Cudf_types.vpkg list * Cudf.package list)
   (** Not strictly a un-installability, Dependency (a,vpkglist,pkglist) is used
@@ -35,9 +27,6 @@ type reason =
   (** Conflict (a,b,vpkg) means that the package [a] is in conflict
       with package [b] because of vpkg *)
 
-(** The request provided to the solver *)
-type request = Cudf.package list
-
 (** The result of an installability query *)
 type result =
   |Success of (?all:bool -> unit -> Cudf.package list)
@@ -53,10 +42,36 @@ type result =
 (** The aggregated result from the solver *)
 type diagnosis = { result : result; request : request; }
 
+(** {2 Low level Integer Un-installability reasons} *)
+type reason_int =
+  |DependencyInt of (int * Cudf_types.vpkg list * int list)
+  |MissingInt of (int * Cudf_types.vpkg list)
+  |ConflictInt of (int * int * Cudf_types.vpkg)
+
+(** the low-level result. All integers are sat solver indexes and need to be
+    converted using a projection map. Moreover the result also contains the
+    global constraints index that must filtered out before returing the final
+    result to the user *)
+type result_int =
+  |SuccessInt of (?all:bool -> unit -> int list)
+  |FailureInt of (unit -> reason_int list)
+
+type request_int = int list
+
+(** {3 Helpers Functions } *)
+
 (** Turn an integer result into a cudf result *)
 val diagnosis : Common.Util.projection -> Cudf.universe ->
   result_int -> request_int -> diagnosis
 
+(** Turn an integer result into a cudf result *)
+val result : Common.Util.projection -> Cudf.universe -> result_int -> result
+
+(** Turn an integer request into a cudf request *)
+val request : Cudf.universe -> request_int -> Cudf.package list
+
+(** {2 Pretty Priting Functions } *)
+
 module ResultHash : Hashtbl.S with type key = reason
 
 (** Collect aggregate information about not installable packages *)
@@ -80,19 +95,8 @@ val collect : summary -> diagnosis -> unit
 val pp_out_version : Format.formatter -> unit
 
 (** default package pretty printer. *)
-val pp_package : ?source:bool -> ?fields:bool -> Common.CudfAdd.pp -> Format.formatter -> Cudf.package -> unit
-
-(** print a list of cudf dependency. The label specifies the type of 
-    dependency ("depends" by default) *)
-val pp_dependency :
-  Common.CudfAdd.pp ->
-  ?label : string ->
-  Format.formatter ->
-  Cudf.package * Cudf_types.vpkglist -> unit
-
-(** Print the list of dependencies of a package. *)
-val pp_dependencies : Common.CudfAdd.pp ->
-  Format.formatter -> (Cudf.package * Cudf_types.vpkglist) list list -> unit
+val pp_package : ?source:bool -> ?fields:bool -> Common.CudfAdd.pp ->
+  Format.formatter -> Cudf.package -> unit
 
 val pp_list :
   (Format.formatter -> 'a -> unit) -> Format.formatter -> 'a list -> unit
@@ -118,15 +122,6 @@ val pp_summary :
   ?pp : Common.CudfAdd.pp ->
   ?explain : bool -> unit -> Format.formatter -> summary -> unit
 
-val print_error_human :
-  ?prefix:string -> Common.CudfAdd.pp ->
-  Cudf.package -> Format.formatter -> reason list -> unit
-
-(** print a human readable explanation (DEV) *)
-val fprintf_human :
-  ?pp : Common.CudfAdd.pp ->
-  ?prefix : string -> Format.formatter -> diagnosis -> unit
-
 (** [printf fmt d] print the output of the solver in yaml format 
     to the formatter [fmt].
     @param pp cudf package printer
@@ -147,9 +142,7 @@ val printf :
   ?pp : Common.CudfAdd.pp ->
   ?failure : bool -> ?success : bool -> ?explain : bool -> diagnosis -> unit
 
-#ifdef HASOCAMLGRAPH
 (** print the explanation graph in dot format to the standard formatter *)
 val print_dot :
   ?pp : Common.CudfAdd.pp -> ?condense : bool ->
   ?addmissing : bool -> ?dir : string -> diagnosis -> unit
-#endif
