@@ -45,6 +45,28 @@ let f_debian = Filename.concat cudf_dir "debian.cudf"
 let f_dominators_order = Filename.concat test_dir "dominators_order.cudf"
 let f_dominators_cycle = Filename.concat test_dir "dominators_cycle.cudf"
 
+let f_is_consistent_success = "tests/cudf/is_consistent_success.cudf"
+let f_is_consistent_failure = "tests/cudf/is_consistent_failure.cudf"
+
+module NV = struct
+  type t = (Cudf_types.pkgname * Cudf_types.version)
+  let compare = compare
+  let pp_printer fmt (n,v) = Format.fprintf fmt "(\"%s\",%d)" n v
+  let pp_print_sep fmt () = Format.fprintf fmt ";"
+end
+
+module SetNV = OUnitDiff.SetMake(NV);;
+
+module PKG = struct
+  type t = Cudf.package
+  let compare = CudfAdd.compare
+  let pp_printer fmt n =
+    Format.fprintf fmt "(\"%s\",%d)" n.Cudf.package n.Cudf.version
+  let pp_print_sep fmt () = Format.fprintf fmt ";"
+end
+
+module SetPKG = OUnitDiff.SetMake(PKG);;
+
 let load_univ f =
   let (_,univ,_) = Cudf_parser.load_from_file f in
   univ
@@ -64,6 +86,34 @@ let engine_conflicts_set = toset f_engine_conflicts
 
 let solver = Depsolver.load universe ;;
 
+(* These tests could be a bit more precise... *)
+let test_is_consistent =
+  "is_consistent" >::: [
+    "success" >:: (fun _ ->
+      let universe = load_univ f_is_consistent_success in
+      let d = Depsolver.is_consistent universe in
+      match d with
+      |{ Diagnostic.result = (Diagnostic.Success f) } -> 
+          (* Diagnostic.printf ~failure:true ~success:true ~explain:true d; *)
+          assert_bool "pass" true
+
+      |{ Diagnostic.result = (Diagnostic.Failure _ ) } ->
+          (* Diagnostic.printf ~failure:true ~success:true ~explain:true d; *)
+          assert_failure "fail"
+    );
+    "failure" >:: (fun _ ->
+      let universe = load_univ f_is_consistent_failure in
+      match Depsolver.is_consistent universe with
+      |{ Diagnostic.result = (Diagnostic.Success f) } -> 
+          (* Diagnostic.printf ~failure:true ~success:true ~explain:true d; *)
+          assert_failure "fail"
+
+      |{ Diagnostic.result = (Diagnostic.Failure _ ) } ->
+          (* Diagnostic.printf ~failure:true ~success:true ~explain:true d; *)
+          assert_bool "pass" true
+    );
+  ]
+
 let test_install =
   "install" >:: (fun _ ->
     let bicycle = Cudf.lookup_package universe ("bicycle", 7) in
@@ -93,55 +143,176 @@ let test_coinst_real =
   )
 
 (* try to coinstall a and b while b has a conflict with c
- * that is declared as keep. Since global_constraints:true
- * this should not be possible *)
+   that is declared as keep. Since global_constraints:true
+   by default this should not be possible *)
 let test_coinst_constraints =
   "coinst constraints" >:: (fun _ ->
-    let a = { Cudf.default_package with 
+    let a1 = { Cudf.default_package with 
       Cudf.package = "a";
-      Cudf.version = 1;
+      version = 1;
     } in
-    let b = { Cudf.default_package with 
+    let b1 = { Cudf.default_package with 
       Cudf.package = "b";
-      Cudf.version = 1;
+      version = 1;
       conflicts = [("c",None)];
     } in
-    let c = { Cudf.default_package with 
+    let c1 = { Cudf.default_package with 
       Cudf.package = "c";
-      Cudf.version = 1;
+      version = 1;
+      installed = true;
       keep = `Keep_package;
     } in
-    let d = { Cudf.default_package with 
+    let d1 = { Cudf.default_package with 
       Cudf.package = "d";
-      Cudf.version = 1;
+      version = 1;
+      installed = true;
       keep = `Keep_version;
     } in
-    let universe = Cudf.load_universe [a;b;c;d] in
-    let d = Depsolver.edos_coinstall ~global_constraints:true universe [a;b] in
+    let universe = Cudf.load_universe [a1;b1;c1;d1] in
+    let d = Depsolver.edos_coinstall universe [a1;b1] in
     assert_bool "pass" (not(Diagnostic.is_solution d))
   )
 
+let test_keep_package =
+  let test installed = 
+    let a1 = { Cudf.default_package with 
+      Cudf.package = "a";
+      version = 1;
+    } in
+    let b1 = { Cudf.default_package with 
+      Cudf.package = "b";
+      version = 1;
+      depends = [[("c",Some (`Gt, 1))]]
+    } in
+    let c1 = { Cudf.default_package with 
+      Cudf.package = "c";
+      version = 1;
+      installed = installed;
+      keep = `Keep_package;
+      conflicts = [("c",None)];
+    } in
+    let c2 = { Cudf.default_package with 
+      Cudf.package = "c";
+      version = 2;
+      conflicts = [("c",None)];
+    } in
+    let universe = Cudf.load_universe [a1;b1;c1;c2] in
+    let d = Depsolver.edos_install universe b1 in
+    SetPKG.assert_equal 
+      (SetPKG.of_list [b1;c2])
+      (SetPKG.of_list (Diagnostic.get_installationset d))
+  in
+  "keep package" >::: [
+    "not installed" >:: (fun _ -> test false);
+    "installed" >:: (fun _ -> test true);
+  ]
+
+let test_keep_version =
+  let test installed =
+    let a1 = { Cudf.default_package with 
+      Cudf.package = "a";
+      version = 1;
+    } in
+    let b1 = { Cudf.default_package with 
+      Cudf.package = "b";
+      version = 1;
+      depends = [[("c",Some (`Gt, 1))]]
+    } in
+    let c1 = { Cudf.default_package with 
+      Cudf.package = "c";
+      version = 1;
+      conflicts = [("c",None)];
+      installed = installed;
+      keep = `Keep_version;
+    } in
+    let c2 = { Cudf.default_package with 
+      Cudf.package = "c";
+      version = 2;
+      conflicts = ["c",None]
+    } in
+    let universe = Cudf.load_universe [a1;b1;c1;c2] in
+    let d = Depsolver.edos_install universe b1 in
+    if installed then
+      assert_bool "pass" (not(Diagnostic.is_solution d))
+    else
+      SetPKG.assert_equal 
+        (SetPKG.of_list [b1;c2])
+        (SetPKG.of_list (Diagnostic.get_installationset d))
+  in
+  "keep version" >::: [
+    "installed" >:: (fun _ -> test true);
+    "not installed" >:: (fun _ -> test false)
+  ]
+
+
+let test_keep_broken =
+  "keep package broken" >:: (fun _ -> 
+    let a1 = { Cudf.default_package with 
+      Cudf.package = "a";
+      depends = [[("b",None)]];
+      installed = true;
+      keep = `Keep_package;
+    } in
+    let a2 = { Cudf.default_package with 
+      Cudf.package = "c";
+    } in
+    let universe = Cudf.load_universe [a1;a2] in
+    let d = Depsolver.edos_install universe a2 in
+    assert_bool "pass" (not(Diagnostic.is_solution d))
+  ) 
+
+let test_keep_multi =
+  "keep multi" >:: (fun _ -> 
+    let a1 = { Cudf.default_package with 
+      Cudf.package = "a";
+      version = 1;
+      installed = true;
+      keep = `Keep_package;
+    } in
+    let a2 = { Cudf.default_package with 
+      Cudf.package = "a";
+      version = 2;
+      conflicts = [("c",None)];
+      installed = true;
+      keep = `Keep_package;
+    } in
+    let c1 = { Cudf.default_package with 
+      Cudf.package = "c";
+    } in
+    let universe = Cudf.load_universe [a1;a2;c1] in
+    let d = Depsolver.edos_install universe c1 in
+    SetPKG.assert_equal 
+      (SetPKG.of_list [a1;c1])
+      (SetPKG.of_list (Diagnostic.get_installationset d))
+  ) 
+
 (* Like above but since the default for global_constraints is false
  * then a and b can be co-installed *)
 let test_coinst_negative_constraints =
-  "coinst constraints" >:: (fun _ ->
-    let a = { Cudf.default_package with 
+  "coinst negative constraints" >:: (fun _ ->
+    let a1 = { Cudf.default_package with
       Cudf.package = "a";
-      Cudf.version = 1;
+      version = 1;
     } in
-    let b = { Cudf.default_package with 
+    let b1 = { Cudf.default_package with
       Cudf.package = "b";
-      Cudf.version = 1;
+      version = 1;
       conflicts = [("c",None)];
     } in
-    let c = { Cudf.default_package with 
+    let c1 = { Cudf.default_package with
       Cudf.package = "c";
-      Cudf.version = 1;
+      version = 1;
+      installed = true;
       keep = `Keep_package;
     } in
-    let universe = Cudf.load_universe [a;b;c] in
-    let d = Depsolver.edos_coinstall universe [a;b] in
-    assert_bool "pass" (Diagnostic.is_solution d)
+    let universe = Cudf.load_universe [a1;b1;c1] in
+    let d = Depsolver.edos_coinstall universe [a1;b1] in
+    assert_bool "pass" (not(Diagnostic.is_solution d))
+    (*
+    SetPKG.assert_equal 
+      (SetPKG.of_list [a1;b1])
+      (SetPKG.of_list (Diagnostic.get_installationset d))
+  *)
   )
 
 let test_coinst_prod = 
@@ -175,43 +346,6 @@ let test_coinst_prod =
     assert_equal (List.sort resl) (List.sort res_expected)
   )
 
-let test_essential_broken =
-  "essential broken" >:: (fun _ -> 
-    let pkg1 = { Cudf.default_package with 
-      Cudf.package = "a";
-      Cudf.depends = [[("b",None)]];
-      Cudf.keep = `Keep_package;
-    } in
-    let pkg2 = { Cudf.default_package with 
-      Cudf.package = "c";
-    } in
-    let universe = Cudf.load_universe [pkg1;pkg2] in
-    let d = Depsolver.edos_install ~global_constraints:true universe pkg2 in
-    assert_bool "pass" (not(Diagnostic.is_solution d))
-  ) 
-
-let test_essential_multi =
-  "essential multi" >:: (fun _ -> 
-    let pkg1a = { Cudf.default_package with 
-      Cudf.package = "a";
-      version = 1;
-      installed = true;
-      keep = `Keep_package;
-    } in
-    let pkg1b = { Cudf.default_package with 
-      Cudf.package = "a";
-      version = 2;
-      conflicts = [("c",None)];
-      keep = `Keep_package;
-    } in
-    let pkg2 = { Cudf.default_package with 
-      Cudf.package = "c";
-    } in
-    let universe = Cudf.load_universe [pkg1a;pkg1b;pkg2] in
-    let d = Depsolver.edos_install ~global_constraints:true universe pkg2 in
-    assert_bool "pass" (Diagnostic.is_solution d)
-  ) 
-
 (* debian testing 18/11/2009 *)
 let test_distribcheck =
   "distribcheck" >:: (fun _ -> 
@@ -235,21 +369,20 @@ let test_selfprovide =
 
 let test_dependency_closure = 
   "dependency closure" >:: (fun _ -> 
-    let dependency_set = toset f_dependency in
+    let (_,pl,_) = Cudf_parser.parse_from_file f_dependency in
     let car = Cudf.lookup_package universe ("car",1) in
     let l = Depsolver.dependency_closure universe [car] in
-    (* List.iter (fun pkg -> print_endline (CudfAdd.print_package pkg)) l; *)
-    let set = List.fold_right S.add l S.empty in
-    assert_equal true (S.equal dependency_set set)
+    SetPKG.assert_equal 
+      (SetPKG.of_list pl)
+      (SetPKG.of_list l)
   )
 
 let test_conjunctive_dependency_closure =
   "dependency closure conjunctive" >:: (fun _ ->
     List.iter (fun pkg ->
+      (*Printf.eprintf "%s\n%!" (CudfAdd.string_of_package pkg);*)
       let dcl = Depsolver.dependency_closure ~conjunctive:true universe [pkg] in
-(*      print_endline (CudfAdd.print_package pkg);
-      List.iter (fun pkg -> print_endline (CudfAdd.print_package pkg)) dcl;
-      print_newline (); *)
+      (*List.iter (fun pkg -> Printf.eprintf "%s\n%!" (CudfAdd.string_of_package pkg)) dcl;*)
       let d = Depsolver.edos_coinstall universe dcl in
       match d.Diagnostic.result with
       |Diagnostic.Success _ -> assert_bool "pass" true
@@ -262,17 +395,13 @@ let test_conjunctive_dependency_closure =
 
 let test_conj_dependency = 
   "conjunctive dependency closure" >:: (fun _ -> 
-    let conj_dependency_set = toset f_conj_dependency in
+    let (_,pl,_) = Cudf_parser.parse_from_file f_conj_dependency in
     let pkg = Cudf.lookup_package universe ("bicycle",7) in
     let g = Strongdeps.conjdeps universe [pkg] in
     let l = Defaultgraphs.PackageGraph.conjdeps g pkg in
-    (*
-    List.iter (fun pkg ->
-      print_endline (CudfAdd.string_of_package pkg)
-      ) l;
-    *)
-    let set = List.fold_right S.add l S.empty in
-    assert_equal true (S.equal conj_dependency_set set)
+    SetPKG.assert_equal 
+      (SetPKG.of_list pl)
+      (SetPKG.of_list l)
   )
 
 let test_reverse_dependencies =
@@ -283,11 +412,10 @@ let test_reverse_dependencies =
     let battery = Cudf.lookup_package universe ("battery",3) in
     let h = Depsolver.reverse_dependencies universe in
     let l = CudfAdd.Cudf_hashtbl.find h battery in
-    let set = List.fold_right S.add l S.empty in
-    let rev_dependency_set =
-      List.fold_right S.add [car;electric_engine1;electric_engine2] S.empty
-    in
-    assert_equal true (S.equal rev_dependency_set set)
+    let pl = [car;electric_engine1;electric_engine2] in
+    SetPKG.assert_equal 
+      (SetPKG.of_list pl)
+      (SetPKG.of_list l)
   )
 
 let test_reverse_dependency_closure =
@@ -297,23 +425,12 @@ let test_reverse_dependency_closure =
     let window = Cudf.lookup_package universe ("window",3) in
     let door = Cudf.lookup_package universe ("door",2) in
     let l = Depsolver.reverse_dependency_closure universe [glass] in
-    let set = List.fold_right S.add l S.empty in
-    let rev_dependency_set =
-      List.fold_right S.add [car;glass;door;window] S.empty
-    in
-    assert_equal true (S.equal rev_dependency_set set)
+    let pl = [car;glass;door;window] in
+    SetPKG.assert_equal 
+      (SetPKG.of_list pl)
+      (SetPKG.of_list l)
   )
 
-(* XXX this is the same function in Depsolver.conv that is
- * not exposed in the mli *) 
-let conv solver = function
-  |Depsolver_int.Success(f_int) ->
-      Diagnostic.SuccessInt(fun ?all () ->
-        List.map solver.Depsolver_int.map#inttovar (f_int ())
-      )
-  |Depsolver_int.Failure(r) -> Diagnostic.FailureInt(r)
-;;
-
 (*
 let test_ =
   "" >:: (fun _ ->
@@ -389,6 +506,11 @@ let test_depclean =
       );
       ]
     in
+    (*
+    SetPKG.assert_equal 
+      (SetPKG.of_list expected)
+      (SetPKG.of_list res)
+  *)
     assert_equal (List.sort res) (List.sort expected)
   )
 ;;
@@ -402,8 +524,8 @@ let test_depsolver =
     test_coinst_constraints ;
     test_coinst_negative_constraints ;
     test_trim ;
-    test_essential_broken ;
-    test_essential_multi ;
+    test_keep_broken ;
+    test_keep_multi ;
     test_distribcheck ;
     test_selfprovide ;
     test_dependency_closure ;
@@ -412,11 +534,22 @@ let test_depsolver =
     test_reverse_dependency_closure ;
     test_conjunctive_dependency_closure ;
     test_depclean ;
+    test_is_consistent;
+    test_keep_package;
+    test_keep_version;
   ]
 
-let solution_set =
-  let (_,pl,_) = Cudf_parser.parse_from_file f_legacy_sol in
-  List.fold_right S.add pl S.empty
+module PKGTUPLE = struct
+  type t = Cudf.package * Cudf.package
+  let compare = Pervasives.compare
+  let pp_printer fmt (p,q) =
+    Format.fprintf fmt "%s -> %s" 
+      (CudfAdd.string_of_package p)
+      (CudfAdd.string_of_package q)
+  let pp_print_sep fmt () = Format.fprintf fmt ";"
+end
+
+module SetPKGTUPLE = OUnitDiff.SetMake(PKGTUPLE);;
 
 let test_strong ?(transitive=true) file ?(checkonly=[]) l =
   let module G = Defaultgraphs.PackageGraph.G in
@@ -436,16 +569,9 @@ let test_strong ?(transitive=true) file ?(checkonly=[]) l =
       (p,q)
     ) l
   in
-  (*
-  if not((List.sort sdedges) = (List.sort testedges)) then
-    List.iter (fun (p,q) -> 
-      Printf.eprintf "%s -> %s\n" 
-      (CudfAdd.string_of_package p)
-      (CudfAdd.string_of_package q)
-    ) sdedges
-  ;
-  *)
-  assert_equal (List.sort sdedges) (List.sort testedges)
+  SetPKGTUPLE.assert_equal
+    (SetPKGTUPLE.of_list testedges)
+    (SetPKGTUPLE.of_list sdedges)
 
 let test_strongcfl file l =
   let universe = load_univ file in
@@ -463,16 +589,10 @@ let test_strongcfl file l =
       (p,q)
     ) l
   in
-  (*
-  if not((List.sort scedges) = (List.sort testedges)) then
-    List.iter (fun (p,q) -> 
-      Printf.eprintf "%s <-> %s\n" 
-      (CudfAdd.string_of_package p)
-      (CudfAdd.string_of_package q)
-    ) scedges
-  ;
-  *)
-  assert_equal (List.sort scedges) (List.sort testedges)
+  SetPKGTUPLE.assert_equal
+    (SetPKGTUPLE.of_list testedges)
+    (SetPKGTUPLE.of_list scedges)
+;;
 
 let strongdep_simple =
   "strongdep simple" >:: (fun _ ->
@@ -520,6 +640,7 @@ let strongdep_conj =
     in
     test_strong f_strongdeps_conj edge_list
   )
+;;
 
 let strongdep_deep_dsj =
   "strongdep deep disj" >:: (fun _ ->
@@ -594,6 +715,19 @@ let test_dependency_graph =
     ()
   )
 
+module Edge = struct
+  open Defaultgraphs.PackageGraph
+  type t = G.edge
+  let compare = G.E.compare 
+  let pp_printer fmt e =
+    Format.fprintf fmt "%s -> %s" 
+      (CudfAdd.string_of_package (G.E.src e))
+      (CudfAdd.string_of_package (G.E.dst e))
+  let pp_print_sep fmt () = Format.fprintf fmt ";"
+end
+
+module SetEDGE = OUnitDiff.SetMake(Edge);;
+
 let test_dominators_tarjan_order =
   "dominators tarjan order" >:: (fun _ ->
     let universe = load_univ f_dominators_order in
@@ -602,20 +736,23 @@ let test_dominators_tarjan_order =
     let edges = ["quebec","romeo"] in
     let edges_packages = 
       List.map (fun (p,q) -> 
-        Cudf.lookup_package universe (p,1),
-        Cudf.lookup_package universe (q,1)
+        let pkg_p = Cudf.lookup_package universe (p,1) in
+        let pkg_q = Cudf.lookup_package universe (q,1) in
+        (pkg_p,pkg_q)
       ) edges
     in
-    let size = Defaultgraphs.PackageGraph.G.nb_edges dg in
+    let edges_expected = List.map (fun (p,q) ->
+      Defaultgraphs.PackageGraph.G.E.create p () q
+      ) edges_packages
+    in
     let all = 
       List.map (fun (p,q) ->
         Defaultgraphs.PackageGraph.G.find_all_edges dg p q
       ) edges_packages 
     in
-    List.iter (fun l ->
-      assert_equal ~msg:"too many edges" (List.length l) 1
-    ) all ;
-    assert_equal ~msg:"too many edges in the graph" size 1
+    SetEDGE.assert_equal
+      (SetEDGE.of_list edges_expected)
+      (SetEDGE.of_list (List.flatten all))
   )
 
 let test_dominators_tarjan_cycle =
@@ -635,22 +772,23 @@ let test_dominators_tarjan_cycle =
     in
     let edges_packages = 
       List.map (fun (p,q) -> 
-        try
-          Cudf.lookup_package dom_univ p,
-          Cudf.lookup_package dom_univ q
-        with Not_found -> failwith (Printf.sprintf "(%s,%s) not found" (fst p) (fst q))
+        let pkg_p = Cudf.lookup_package dom_univ p in
+        let pkg_q = Cudf.lookup_package dom_univ q in
+        (pkg_p,pkg_q)
       ) edges
     in
-    let size = Defaultgraphs.PackageGraph.G.nb_edges dg in
+    let edges_expected = List.map (fun (p,q) ->
+      Defaultgraphs.PackageGraph.G.E.create p () q
+      ) edges_packages
+    in
     let all = 
       List.map (fun (p,q) ->
         Defaultgraphs.PackageGraph.G.find_all_edges dg p q
       ) edges_packages 
     in
-    List.iter (fun l ->
-      assert_equal ~msg:"too many edges" (List.length l) 1
-    ) all ;
-    assert_equal ~msg:"too many edges in the graph" size (List.length edges)
+    SetEDGE.assert_equal
+      (SetEDGE.of_list edges_expected)
+      (SetEDGE.of_list (List.flatten all))
   )
 
 let test_dominators_tarjan_legacy =
@@ -673,22 +811,23 @@ let test_dominators_tarjan_legacy =
     in
     let edges_packages = 
       List.map (fun (p,q) -> 
-        try
-          Cudf.lookup_package dom_univ p,
-          Cudf.lookup_package dom_univ q
-        with Not_found -> failwith (Printf.sprintf "(%s,%s) not found" (fst p) (fst q))
+        let pkg_p = Cudf.lookup_package dom_univ p in
+        let pkg_q = Cudf.lookup_package dom_univ q in
+        (pkg_p,pkg_q)
       ) edges
     in
-    let size = Defaultgraphs.PackageGraph.G.nb_edges dg in
+    let edges_expected = List.map (fun (p,q) ->
+      Defaultgraphs.PackageGraph.G.E.create p () q
+      ) edges_packages
+    in
     let all = 
       List.map (fun (p,q) ->
         Defaultgraphs.PackageGraph.G.find_all_edges dg p q
       ) edges_packages 
     in
-    List.iter (fun l ->
-      assert_equal ~msg:"too many edges" (List.length l) 1
-    ) all ;
-    assert_equal ~msg:"too many edges in the graph" size (List.length edges)
+    SetEDGE.assert_equal
+      (SetEDGE.of_list edges_expected)
+      (SetEDGE.of_list (List.flatten all))
   )
 
 
@@ -697,12 +836,26 @@ let test_dominators_direct_order =
     let universe = load_univ f_dominators_order in
     let g = Strongdeps.strongdeps_univ ~transitive:false universe in
     let dg = Dominators.dominators_direct g in
-    let romeo = Cudf.lookup_package universe ("romeo",1) in
-    let quebec = Cudf.lookup_package universe ("quebec",1) in
-    let size = Defaultgraphs.PackageGraph.G.nb_edges dg in
-    let all = Defaultgraphs.PackageGraph.G.find_all_edges dg quebec romeo in
-    assert_equal ~msg:"too many edges between quebec and romeo" (List.length all) 1;
-    assert_equal ~msg:"too many edges in the graph" size 1
+    let edges = [("quebec",1),("romeo",1)] in
+    let edges_packages = 
+      List.map (fun (p,q) -> 
+        let pkg_p = Cudf.lookup_package universe p in
+        let pkg_q = Cudf.lookup_package universe q in
+        (pkg_p,pkg_q)
+      ) edges
+    in
+    let edges_expected = List.map (fun (p,q) ->
+      Defaultgraphs.PackageGraph.G.E.create p () q
+      ) edges_packages
+    in
+    let all = 
+      List.map (fun (p,q) ->
+        Defaultgraphs.PackageGraph.G.find_all_edges dg p q
+      ) edges_packages 
+    in
+    SetEDGE.assert_equal
+      (SetEDGE.of_list edges_expected)
+      (SetEDGE.of_list (List.flatten all))
   )
 
 let test_dominators =
