@@ -30,7 +30,6 @@ module Options = struct
 
   let dump = StdOpt.store_true ()
   let noop = StdOpt.store_true ()
-  let human = StdOpt.store_true ()
   let solver = StdOpt.str_option ()
   let criteria = StdOptions.criteria_option ()
   let criteria_plain = StdOpt.str_option ()
@@ -47,20 +46,23 @@ module Options = struct
   add options ~short_name:'c' ~long_name:"criteria" ~help:"optimization criteria in extended MISC syntax" criteria;
   add options ~long_name:"criteria-plain" ~help:"optimization criteria passed unmangled to the solver" criteria_plain;
   add options ~short_name:'e' ~long_name:"explain" ~help:"print installation summary" explain;
-  add options ~long_name:"human" ~help:"print human readable installation errors" human;
   add options ~long_name:"native-arch" ~help:"Native architecture" native_arch;
   add options ~long_name:"foreign-archs" ~help:"Foreign architectures" foreign_archs;
 
 end
 
+let doseexit () =
+  if OptParse.Opt.is_set Options.solver then (
+    Format.printf "Error: %s" (OptParse.Opt.get Options.solver);
+    if OptParse.Opt.is_set Options.criteria_plain then
+      Format.printf " \"%s\"" (OptParse.Opt.get Options.criteria_plain);
+    Format.printf "@."
+  )
+;;
+
 let fatal fmt =
   Printf.kprintf (fun s ->
-    if OptParse.Opt.is_set Options.solver then (
-      Format.printf "Error: %s" (OptParse.Opt.get Options.solver);
-      if OptParse.Opt.is_set Options.criteria_plain then
-        Format.printf " \"%s\"" (OptParse.Opt.get Options.criteria_plain);
-      Format.printf "@."
-    );
+    doseexit ();
     Format.printf "Message: %s@." s;
     exit 1
   ) fmt
@@ -391,14 +393,17 @@ let main () =
           | Some fieldvalue -> begin
               let matches = match compiledre with
                 | None when String.exists fieldvalue plain -> true (* plain text search *)
-                | Some compiledre when Pcre.pmatch ~rex:compiledre fieldvalue -> true (* regex match *)
+                | Some compiledre
+                  when Pcre.pmatch ~rex:compiledre fieldvalue -> true (* regex match *)
                 | _ -> false
               in
               match matches with
-              | true -> { acc with Cudf.pkg_extra = (cudffield, `Int 1) :: acc.Cudf.pkg_extra }
+              | true ->
+                  { acc with Cudf.pkg_extra = (cudffield, `Int 1) :: acc.Cudf.pkg_extra }
               | false -> acc (* no match *)
             end
-        ) regexfields p in
+        ) regexfields p 
+      in
       if not(Hashtbl.mem univ (p.Cudf.package,p.Cudf.version)) then begin
         Hashtbl.add univ (p.Cudf.package,p.Cudf.version) pkg;
         Some p
@@ -466,23 +471,33 @@ let main () =
 
   (* do nothing. *)
   if OptParse.Opt.get Options.noop then begin
-    info "Noop & Exit."; exit(0)
+    info "Noop & Exit.";
+    exit 0
   end;
 
   let exec_pat = fst (parse_solver_spec (Filename.concat solver_dir solver)) in
 
   let solpre,soluniv = 
-    let from_cudf (p,v) = Debian.Debcudf.get_real_version tables (p,v) in
-    let pp = CudfAdd.pp from_cudf in
-    let explain = OptParse.Opt.get Options.human in
-    match Algo.Depsolver.check_request ~cmd:exec_pat ~criteria ~explain cudf with
+    let dummy = { Algo.Depsolver.dummy_request with
+      Cudf.depends =
+        List.map (fun (_,pkglist) ->
+          List.map (fun pkg ->
+            (pkg.Cudf.package,Some(`Eq,pkg.Cudf.version))
+          ) pkglist
+        ) (Debcudf.get_essential tables) }
+    in
+    let explain = false in
+    match Algo.Depsolver.check_request ~cmd:exec_pat ~dummy ~criteria ~explain cudf with
     |Algo.Depsolver.Error s -> fatal "%s" s
-    |Algo.Depsolver.Unsat None ->
-      fatal "(UNSAT) No Solutions according to the given preferences"
+    |Algo.Depsolver.Unsat None -> begin
+      doseexit ();
+      Format.printf "Message: (UNSAT) No Solutions according to the given preferences@.";
+      exit 0
+    end
     |Algo.Depsolver.Unsat Some d -> begin
-      Format.printf "Error: (UNSAT) No Solutions according to the given preferences@.";
-      Format.printf "%a@." (Algo.Diagnostic.fprintf_human ~prefix:"Message: " ~pp) d;
-      exit 1
+      doseexit ();
+      Format.printf "Message: (UNSAT) No Solutions according to the given preferences@.";
+      exit 0
     end
     |Algo.Depsolver.Sat s -> s
   in
@@ -500,10 +515,19 @@ let main () =
   Util.Timer.start timer3;
   let empty = ref true in
   let cache = CudfAdd.Cudf_hashtbl.create 1023 in
+  (* In Debian (and thus for apt and dpkg), packages can only be installed in a
+   * single version at a time. Thus, any up or downgrades always implicitly
+   * remove the old version. Therefore, if a package installation request just
+   * changes the version of the package, we remember this in a hash table such
+   * that we do not generate a removal request for that package as well. As a
+   * result, every package (name) only shows up exactly once in the solution.
+   * Package removals are not explicitly shown for upgrades. *)
+  let notremoved = Util.StringHashtbl.create 1023 in
 
   let (install,remove) = CudfDiff.make_solution ~universe ~solution:soluniv in
   CudfAdd.Cudf_set.iter (fun pkg ->
     CudfAdd.Cudf_hashtbl.add cache pkg ();
+    Util.StringHashtbl.add notremoved pkg.Cudf.package ();
     Format.printf "Install: %a@." pp_pkg (pkg,univ)
   ) install;
 
@@ -521,7 +545,8 @@ let main () =
   ) cudf_request.Cudf.install;
 
   CudfAdd.Cudf_set.iter (fun p ->
-    Format.printf "Remove: %a@." pp_pkg (p,univ)
+    if Util.StringHashtbl.mem notremoved p.Cudf.package then ()
+    else Format.printf "Remove: %a@." pp_pkg (p,univ)
   ) remove;
 
   Util.Timer.stop timer3 ();
