@@ -92,11 +92,9 @@ let version_of_target getv = function
   |`Lo v |`In (_,v) -> (getv v) - 1
 ;;
 
-let timer = Util.Timer.create "Solver"
-
 (* the repository should contain only the most recent version of each
    package *)
-let future ?options ?(checklist=[]) repository =
+let future ~options ?(checklist=[]) repository =
 
   let worktable = Hashtbl.create 1024 in
   let version_acc = ref [] in
@@ -155,7 +153,7 @@ let future ?options ?(checklist=[]) repository =
   info "Total Names: %d" (Hashtbl.length worktable);
   info "Total versions: %d" (List.length versionlist);
 
-  let tables = Debian.Debcudf.init_tables ~step:2 ~versionlist repository in
+  let tables = Debian.Debcudf.init_tables ~options ~step:2 ~versionlist repository in
   let getv v = Debian.Debcudf.get_cudf_version tables ("",v) in
 
   let pkgset = 
@@ -165,7 +163,7 @@ let future ?options ?(checklist=[]) repository =
       let acc0 = 
         (* by assumption all packages in a cluster are syncronized *)
         List.fold_left (fun l pkg ->
-          let p = Debian.Debcudf.tocudf ?options tables pkg in
+          let p = Debian.Debcudf.tocudf ~options tables pkg in
           CudfAdd.Cudf_set.add (sync (sn,version,1) p) l
         ) acc0 cluster
       in
@@ -173,7 +171,7 @@ let future ?options ?(checklist=[]) repository =
       List.fold_left (fun acc1 (target,equiv) ->
         incr sync_index;
         List.fold_left (fun acc2 pkg ->
-          let p = Debian.Debcudf.tocudf ?options tables pkg in
+          let p = Debian.Debcudf.tocudf ~options tables pkg in
           let pv = p.Cudf.version in
 
           let target = Debian.Evolution.align pkg#version target in
@@ -207,9 +205,9 @@ let outdated
   ?(explain=false) 
   ?(summary=false)
   ?(checklist=None) 
-  ?options repository =
+  ~options repository =
 
-  let universe, tables = future ?options repository in
+  let universe, tables = future ~options repository in
  
   let universe_size = Cudf.universe_size universe in
   info "Total future: %d" universe_size;
@@ -226,7 +224,7 @@ let outdated
     if OptParse.Opt.is_set Options.checkonly then begin
       List.flatten (
         List.map (fun ((n,a),c) ->
-          let (name,filter) = Debian.Debutil.debvpkg to_cudf ((n,a),c) in
+          let (name,filter) = Pef.Pefcudf.pefvpkg to_cudf ((n,a),c) in
           Cudf.lookup_packages ~filter universe name
         ) (OptParse.Opt.get Options.checkonly)
       )
@@ -234,10 +232,11 @@ let outdated
   in
 
   let pp pkg =
-    let p = 
-      if String.starts_with pkg.Cudf.package "src/" then
-        Printf.sprintf "Source conflict (%s)" pkg.Cudf.package
-      else pkg.Cudf.package
+    let (p,a) = 
+      let (n,a) = Debian.Debcudf.get_real_name pkg.Cudf.package in
+      if String.starts_with n "src/" then
+        (Printf.sprintf "Source conflict (%s)" n,a)
+      else (n,a)
     in
     let v = 
       if pkg.Cudf.version > 0 then begin
@@ -247,8 +246,10 @@ let outdated
           try Cudf.lookup_package_property pkg "number"
           with Not_found ->
             if (pkg.Cudf.version mod 2) = 1 then
-              snd(Debian.Debcudf.get_real_version tables 
-              (pkg.Cudf.package,pkg.Cudf.version))
+              let (_,_,v) =
+                Debian.Debcudf.get_real_version tables 
+                (pkg.Cudf.package,pkg.Cudf.version)
+              in v
             else
               fatal "Real package without Debian Version"
       end
@@ -260,7 +261,7 @@ let outdated
         with Not_found -> None
       ) ["architecture";"source";"sourcenumber";"equivs"]
     in
-    (CudfAdd.decode p,v,l)
+    (p,a,v,l)
   in
 
   let fmt = Format.std_formatter in
@@ -270,17 +271,16 @@ let outdated
   let results = Diagnostic.default_result universe_size in
   let callback d = 
     if summary then Diagnostic.collect results d ;
-    Diagnostic.fprintf ~pp ~failure ~explain fmt d 
+    if failure then
+      Diagnostic.fprintf ~pp ~failure ~explain fmt d 
   in
 
-  Util.Timer.start timer;
   let broken =
     if checklist <> [] then
-      Depsolver.listcheck ~callback ~global_constraints:false universe checklist
+      Depsolver.listcheck ~callback ~explain universe checklist
     else
-      Depsolver.univcheck ~callback ~global_constraints:false universe
+      Depsolver.univcheck ~callback ~explain universe
   in
-  ignore(Util.Timer.stop timer ());
 
   if failure then Format.fprintf fmt "@]@.";
 
@@ -303,8 +303,9 @@ let main () =
 
   StdDebug.enable_debug (OptParse.Opt.get Options.verbose);
   StdDebug.enable_bars (OptParse.Opt.get Options.progress)
-    ["Depsolver_int.univcheck";"Depsolver_int.init_solver"] ;
-  StdDebug.enable_timers (OptParse.Opt.get Options.timers) ["Solver"];
+  ["Depsolver_int.univcheck";"Depsolver_int.init_solver"];
+  StdDebug.enable_timers (OptParse.Opt.get Options.timers)
+    ["Algo.Depsolver.solver"; "Algo.Depsolver.init" ] ;
   StdDebug.all_quiet (OptParse.Opt.get Options.quiet);
 
   let checklist = OptParse.Opt.opt Options.checkonly in
