@@ -13,14 +13,10 @@
 (** This module compute the difference between a cudf universe and a cudf solution *)
 
 open ExtLib
+open CudfAdd
 
-module Cudf_set = CudfAdd.Cudf_set
-module StringSet = CudfAdd.StringSet
-
-type solution = {
-  installed : Cudf_set.t ;
-  removed : Cudf_set.t ;
-}
+type changeset = (Cudf_set.t * Cudf_set.t)
+type difference = (string,changeset) Hashtbl.t
 
 (* the 'package' is always taken from the universe *)
 let to_set univ l =
@@ -30,22 +26,30 @@ let to_set univ l =
   ) Cudf_set.empty l
 ;;
 
-(** [diff univ sol] for each pkgname returns the list of all 
+(* [make_solution univ sol] return a tuple with the set of packages to install and the
+   set of packages to remove *)
+let make_solution ~universe ~solution =
+  let is_installed p = p.Cudf.installed in
+  let before = CudfAdd.to_set (Cudf.get_packages ~filter:is_installed universe) in
+  let after = CudfAdd.to_set (Cudf.get_packages ~filter:is_installed solution) in
+  let install = Cudf_set.diff after before in
+  let remove = Cudf_set.diff before after in
+  (install,remove)
+
+(** [make_difference univ sol] for each pkgname returns the list of all 
     versions that were installed (in the solution) or removed (from the
     universe) *)
-let diff univ sol =
-  let pkgnames = CudfAdd.pkgnames univ in
+let make_difference ~universe ~solution =
+  let pkgnames = CudfAdd.pkgnames universe in
   let h = Hashtbl.create (StringSet.cardinal pkgnames) in
   StringSet.iter (fun pkgname ->
-    let were_installed = to_set univ (Cudf.get_installed univ pkgname) in
-    let are_installed = to_set univ (Cudf.get_installed sol pkgname) in
-    let r = Cudf_set.diff were_installed are_installed in
-    let i = Cudf_set.diff are_installed were_installed in
-    let s = { removed = r ; installed = i } in
-    Hashtbl.add h pkgname s
+    let were_installed = to_set universe (Cudf.get_installed universe pkgname) in
+    let are_installed = to_set universe (Cudf.get_installed solution pkgname) in
+    let remove = Cudf_set.diff were_installed are_installed in
+    let install = Cudf_set.diff are_installed were_installed in
+    Hashtbl.add h pkgname (install,remove)
   ) pkgnames ;
   h
-;;
 
 (**
    [all] : all versions of a package in the universe . 
@@ -53,61 +57,76 @@ let diff univ sol =
    returns a list that contains for each version its status : installed, 
    removed, upgraded, etc
 *)
-type summary_t = {
+type tmp_summary = {
   mutable i : Cudf.package list; (* installed *)
   mutable r : Cudf.package list; (* removed *)
   mutable u : (Cudf.package * Cudf.package) option ; (* upgraded *)
   mutable d : (Cudf.package * Cudf.package) option ; (* downgraded *)
-  mutable nu : Cudf.package list; (* not upgraded *)
+  mutable nc : Cudf.package list; (* not changed *)
 }
 
-(* for one package *)
-let default_summary () = { u = None; d = None ; i = [] ; r = [] ; nu = [] }
-
-let uniqueversion all s =
-  let l = default_summary () in
+let uniqueversion all (install,remove) =
+  let l = { u = None; d = None ; i = [] ; r = [] ; nc = [] } in
   let i = Cudf_set.filter (fun pkg -> pkg.Cudf.installed) all in
-  if (Cudf_set.cardinal i <= 1) && ((Cudf_set.cardinal s.installed) <= 1) then
+  if (Cudf_set.cardinal i <= 1) && ((Cudf_set.cardinal install) <= 1) then
     begin
-      if (Cudf_set.cardinal s.installed) = 1 then begin
+      if (Cudf_set.cardinal install) = 1 then begin
         if (Cudf_set.cardinal i) = 1 then begin
           let np = Cudf_set.choose i in
-          let op = Cudf_set.choose s.installed in
+          let op = Cudf_set.choose install in
           if np.Cudf.version < op.Cudf.version
           then l.u <- Some(np,op)
           else l.d <- Some(op,np)
         end
         else
-          l.i <- Cudf_set.elements s.installed;
+          l.i <- Cudf_set.elements install;
       end else
-        if not (Cudf_set.is_empty s.removed) then
-          l.r <- Cudf_set.elements s.removed;
+        if not (Cudf_set.is_empty remove) then
+          l.r <- Cudf_set.elements remove;
       end
   else begin
-    if not (Cudf_set.is_empty s.removed) then
-      l.r <- Cudf_set.elements s.removed;
-    if not (Cudf_set.is_empty s.installed) then
-      l.i <- Cudf_set.elements s.installed;
+    if not (Cudf_set.is_empty remove) then
+      l.r <- Cudf_set.elements remove;
+    if not (Cudf_set.is_empty install) then
+      l.i <- Cudf_set.elements install;
   end;
+  l.nc <- Cudf_set.elements (Cudf_set.diff i (Cudf_set.union install remove));
   l
 ;;
 
-let summary univ diff =
-  let i = ref [] in
-  let u = ref [] in
-  let d = ref [] in
-  let r = ref [] in
+type summary = {
+  install : Cudf.package list;
+  remove : Cudf.package list;
+  upgrade : (Cudf.package * Cudf.package) list ;
+  downgrade : (Cudf.package * Cudf.package) list ;
+  notchange : Cudf.package list;
+}
+
+(* [summary univ diff] return a tuple with 5 lists containing packages
+ * installed, upgraded, downgraded, removed, not upgraded 
+ * (all packages marked as installed but not in the categories above) *)
+let make_summary univ diff =
+  let i = ref [] 
+  and u = ref [] 
+  and d = ref []
+  and r = ref []
+  and nc = ref [] in
   let names = CudfAdd.pkgnames univ in
   StringSet.iter (fun pkgname ->
     let all = CudfAdd.to_set (Cudf.lookup_packages univ pkgname) in
     let s = Hashtbl.find diff pkgname in
     let l = uniqueversion all s in
-    i := l.i @ !i ; 
-    r := l.r @ !r ; 
+    i := l.i @ !i ;
+    r := l.r @ !r ;
     if not (Option.is_none l.u) then
       u := (Option.get l.u) :: !u;
     if not (Option.is_none l.d) then
       d := (Option.get l.d) :: !d;
+    nc := l.nc @ !nc;
   ) names;
-  (!i,!u,!d,!r)
+  {install = !i;
+   remove = !r;
+   upgrade = !u;
+   downgrade = !d;
+   notchange = !nc }
 ;;
