@@ -76,8 +76,8 @@ let parse_uri =
 module S = CudfAdd.Cudf_set
 
 let (universe,request) =
-  let (_,univ,request) = Cudf_parser.parse_from_file f_legacy in
-  (Cudf.load_universe univ,Option.get request)
+  let (_,pkglist,request) = Cudf_parser.parse_from_file f_legacy in
+  (Cudf.load_universe pkglist,Option.get request)
 
 let toset l =
   List.fold_right (fun e set ->
@@ -101,6 +101,38 @@ let test_who_conflicts =
     assert_equal true (S.equal engine_conflicts_set set)
   )
 
+let cone_cases =
+  let open Cudf in
+  let printer xs =
+    List.fold_left (fun acc x -> Printf.sprintf "%s, %s" acc x.package) "" xs
+  in
+  let univ_ls = Cudf.fold_packages (fun acc x -> x :: acc) [] universe in
+  let cone input = CudfAdd.cone universe input in
+  let pedal_ls = [Cudf.package_by_uid universe 3] in
+  let engine_ls = [Cudf.package_by_uid universe 8] in
+  let engine_cone = [Cudf.package_by_uid universe 18; Cudf.package_by_uid universe 8] in
+  [
+    ("cone universe", univ_ls, (fun c () -> assert_equal (List.length (cone c)) (List.length univ_ls)));
+    ("cone pedal", pedal_ls, (fun c () -> assert_equal (cone c) pedal_ls ~printer:printer));
+    ("cone engine", engine_ls, (fun c () -> assert_equal (cone c) engine_cone ~printer:printer));
+  ]
+
+
+let criteria_parse =
+  let function_to_test crt = Criteria.parse_criteria ("test",(Format822.dummy_loc,crt)) in
+  let printer s = Criteria.to_string s in
+  let returns = returns_result ~printer function_to_test in
+  [
+    ("parse upgrade", "-count(new),-count(removed),-notuptodate(solution)", returns 
+      (List.assoc "upgrade" Criteria.default_criteria));
+    ("parse trendy", "-count(removed),-notuptodate(solution),-unsat_recommends(solution),-count(new)", returns 
+      (List.assoc "trendy" Criteria.default_criteria));
+    ("parse count exact", "-count(solution,APT-Release:=/experimental/)", returns 
+      Criteria_types.([Minimize(Count(Solution,Some("APT-Release",ExactMatch "experimental")))]));
+    ("parse count regexp", "-count(solution,APT-Release:~/stable|unstable/)", returns 
+      Criteria_types.([Minimize(Count(Solution,Some("APT-Release",Regexp "stable|unstable")))]))
+  ]
+
 let test_who_provides =
   "who_provides" >:: (fun _ ->
     let l = CudfAdd.who_provides universe ("electric-engine",None) in
@@ -261,6 +293,34 @@ let criteria_parse =
       Criteria_types.([Minimize(Count(Solution,Some("APT-Release",Regexp "stable|unstable")))]))
   ]
 
+let dummy name version =
+  {Cudf.default_package with
+   Cudf.package = name;
+   version = version;
+  }
+
+module PkgSetTest = OUnitDiff.SetMake (struct
+  type t = Cudf.package
+  let compare = CudfAdd.compare
+  let pp_printer = CudfAdd.pp_package 
+  let pp_print_sep = OUnitDiff.pp_comma_separator
+end)
+
+let returns_result_pkgset function_to_test expected_result =
+  (fun args () -> PkgSetTest.assert_equal (function_to_test args) expected_result)
+
+let latest_cases =
+  let a1,a2,a3,a4 = dummy "a" 1, dummy "a" 2, dummy "a" 3, dummy "a" 4 in
+  let b4,b2 = dummy "b" 4, dummy "b" 2 in
+  let univ = [a1;a2;a3;a4;b4;b2] in
+  let returns = returns_result_pkgset (fun u -> PkgSetTest.of_list (CudfAdd.latest u)) in
+  let returns_nth n = returns_result_pkgset (fun u -> PkgSetTest.of_list (CudfAdd.latest ~n u)) in
+  [
+    ("latest", univ, returns (PkgSetTest.of_list [a4;b4])) ;
+    ("latest 3", univ, returns_nth 3 (PkgSetTest.of_list [a4;a2;a3;b4;b2])) ;
+    ("latest 2", univ, returns_nth 2 (PkgSetTest.of_list [a3;a4;b4;b2])) ;
+  ]
+  
 let make_test_cases triplets =
   List.map ( fun (test_name, input, assert_function) -> test_name >:: assert_function input ) triplets
 
@@ -273,6 +333,8 @@ let all =
     test_shell_lexer;
     "criteria" >::: make_test_cases criteria_to_string;
     "criteria" >::: make_test_cases criteria_parse;
+    "cone" >::: make_test_cases cone_cases;
+    "cudfadd" >::: make_test_cases latest_cases;
   ]
 
 let main () =
