@@ -23,6 +23,8 @@ val memo : ('a -> 'b) -> 'a -> 'b
 
 val timestamp : unit -> string
 
+val max32int : int
+
 (** Debug, ProgressBars, Timers and Loggers *)
 
 type label = string
@@ -159,6 +161,11 @@ module IntPairHashtbl : Hashtbl.S with type key = int * int
 module StringHashtbl : Hashtbl.S with type key = string
 module StringPairHashtbl : Hashtbl.S with type key = string * string
 
+(** hash consing for strings *)
+val hashcons: string StringHashtbl.t -> string -> string
+val hits : int ref
+val miss : int ref
+
 val range : int -> int -> int list
 
 val string_of_list :
@@ -175,6 +182,7 @@ class type projection =
     method inttovar : int -> int
     (** given a sat solver variable return a package id *)
     method vartoint : int -> int
+    method size : int
   end
 
 (** identity projection *)
