@@ -62,17 +62,17 @@ let default_release = {
 
 let parse_release_stanza fname par = {
   fname = Filename.basename (fname);
-  origin = Packages.parse_s ~opt:"" Packages.parse_string "Origin" par;
-  label = Packages.parse_s ~opt:"" Packages.parse_string "Label" par;
-  suite = Packages.parse_s ~opt:"" Packages.parse_string "Suite" par;
-  version = Packages.parse_s ~opt:"" Packages.parse_string "Version" par;
-  codename = Packages.parse_s ~opt:"" Packages.parse_string "Codename" par;
-  date = Packages.parse_s ~opt:"" Packages.parse_string "Date" par;
-  architecture = Packages.parse_s ~opt:"" Packages.parse_string "Architectures" par;
-  component = Packages.parse_s ~opt:"" Packages.parse_string "Components" par;
-  notauto = Packages.parse_s ~opt:false Packages.parse_bool "NotAutomatic" par;
-  autoup = Packages.parse_s ~opt:false Packages.parse_bool "ButAutomaticUpgrades" par;
-  description = Packages.parse_s ~opt:"" Packages.parse_string "Description" par;
+  origin = Pef.Packages.parse_s ~default:"" Pef.Packages.parse_string "Origin" par;
+  label = Pef.Packages.parse_s ~default:"" Pef.Packages.parse_string "Label" par;
+  suite = Pef.Packages.parse_s ~default:"" Pef.Packages.parse_string "Suite" par;
+  version = Pef.Packages.parse_s ~default:"" Pef.Packages.parse_string "Version" par;
+  codename = Pef.Packages.parse_s ~default:"" Pef.Packages.parse_string "Codename" par;
+  date = Pef.Packages.parse_s ~default:"" Pef.Packages.parse_string "Date" par;
+  architecture = Pef.Packages.parse_s ~default:"" Pef.Packages.parse_string "Architectures" par;
+  component = Pef.Packages.parse_s ~default:"" Pef.Packages.parse_string "Components" par;
+  notauto = Pef.Packages.parse_s ~default:false Pef.Packages.parse_bool "NotAutomatic" par;
+  autoup = Pef.Packages.parse_s ~default:false Pef.Packages.parse_bool "ButAutomaticUpgrades" par;
+  description = Pef.Packages.parse_s ~default:"" Pef.Packages.parse_string "Description" par;
   md5sums = [];
   sha1 = [];
   sha256 = []
