@@ -12,10 +12,14 @@
 (*  library, see the COPYING file for more information.                   *)
 (**************************************************************************)
 
+module Pcre = Re_pcre
+
 open ExtLib
 open Common
 
-include Util.Logging(struct let label = __FILE__ end) ;;
+#define __label __FILE__
+let label =  __label ;;
+include Util.Logging(struct let label = label end) ;;
 
 type name = string
 type version = string
@@ -43,7 +47,7 @@ module Set = Set.Make(struct
 end)
 
 let input_raw_priv parse_packages files =
-  let timer = Util.Timer.create __FILE__ in
+  let timer = Util.Timer.create __label in
   Util.Timer.start timer;
   if List.length files > 1 then info "Merging input lists" ;
   let s =
@@ -103,10 +107,11 @@ let parse_packages_in f ch =
   parse_packages_rec [] ch
 
 let parse_packages f filename =
-  let ch = Input.open_file filename in
-  let l = parse_packages_rec [] ch in
-  Input.close_ch ch;
-  l
+  try
+    let ch = Input.open_file filename in
+    let l = parse_packages_rec [] ch in
+    Input.close_ch ch; l
+  with Input.File_empty -> []
 
 let input_raw ?(extras=[]) files = input_raw_priv parse_packages files
 
