@@ -21,35 +21,68 @@ let label =  __label ;;
 include Util.Logging(struct let label = label end) ;;
 
 type tables = {
+  strings_table : string Util.StringHashtbl.t;
   versions_table : (string, string list) Hashtbl.t;
   reverse_table : ((string * int), string) Hashtbl.t
 }
 
 let create n = {
+  strings_table = Util.StringHashtbl.create n;
   versions_table = Hashtbl.create n;
   reverse_table = Hashtbl.create n;
 }
 
+(*
 type lookup = {
-  from_cudf : Cudf.package -> (string * string);
+  from_cudf : Cudf.package -> (string * string option * string);
   to_cudf : (string * string) -> Cudf.package
 }
+*)
 
 let clear tables =
+  Util.StringHashtbl.clear tables.strings_table;
   Hashtbl.clear tables.versions_table;
   Hashtbl.clear tables.reverse_table
 ;;
 
-let init_versions_table table =
+let pefcudf_op = function
+  |("<<" | "<") -> `Lt
+  |(">>" | ">") -> `Gt
+  |"<=" -> `Leq
+  |">=" -> `Geq
+  |"=" -> `Eq
+  |"!=" -> `Neq
+  |c -> fatal "Unknown operator: %s" c
+
+let pefcudf_constr = function
+  |None -> None
+  |Some("ALL",_) -> None
+  |Some(c,v) -> Some(pefcudf_op c,v)
+
+(** convert a pef constraint into a cudf constraint *)
+let pefvpkg to_cudf vpkgname =
+  let constr n = function
+    |None -> None
+    |Some(op,v) -> Some(pefcudf_op op,snd(to_cudf (n,v)))
+  in
+  match vpkgname with
+  |((n,None),c) -> (CudfAdd.encode n,constr n c)
+  |((n,Some ("any"|"native")),c) -> (CudfAdd.encode n,constr n c)
+  |((n,Some a),c) -> (CudfAdd.encode (n^":"^a),constr n c)
+
+
+let init_versions_table tables table =
   let add name version =
     try
+      let version = Util.hashcons tables.strings_table version in
       let l = Hashtbl.find table name in
       Hashtbl.replace table name (version::l)
-    with Not_found -> Hashtbl.add table name [version]
+    with Not_found ->
+      Hashtbl.add table name [version]
   in
   let conj_iter =
     List.iter (fun ((name,_),sel) ->
-      match CudfAdd.cudfop sel with
+      match pefcudf_constr sel with
       |None -> ()
       |Some(_,version) -> add name version
     ) 
@@ -57,7 +90,7 @@ let init_versions_table table =
   let cnf_iter = 
     List.iter (fun disjunction ->
       List.iter (fun ((name,_),sel) ->
-        match CudfAdd.cudfop sel with
+        match pefcudf_constr sel with
         |None -> ()
         |Some(_,version) -> add name version
       ) disjunction
@@ -100,7 +133,7 @@ let init_tables compare pkglist =
   let n = 2 * List.length pkglist in
   let tables = create n in 
   let temp_versions_table = Hashtbl.create n in
-  let ivt = init_versions_table temp_versions_table in
+  let ivt = init_versions_table tables temp_versions_table in
 
   List.iter (fun pkg -> ivt pkg) pkglist ;
 
@@ -116,22 +149,30 @@ let init_tables compare pkglist =
 (* pef -> cudf . versions start from 1 *)
 let get_cudf_version tables (package,version) =
   try
+    let version = Util.hashcons tables.strings_table version in
     let l = Hashtbl.find tables.versions_table package in
     let i = fst(List.findi (fun i a -> a = version) l) in
     Hashtbl.replace tables.reverse_table (CudfAdd.encode package,i+1) version;
     i+1
   with Not_found -> (
-    warning "Cannot find cudf version for %s (= %s)" (CudfAdd.decode package) version;
+    warning "Cannot find cudf version for pef package %s (= %s)" package version;
     raise Not_found
   )
 
 (* cudf -> pef *)
 let get_real_version tables (p,i) =
-  try Hashtbl.find tables.reverse_table (p,i)
-  with Not_found -> (
-    warning "Cannot find real version for %s (= %d)" p i;
-    raise Not_found
-  )
+  let n = 
+    try fst(ExtString.String.split p "%3a")
+    with Invalid_string -> p
+  in
+  if i = Util.max32int || i = Util.max32int - 1 then
+    (CudfAdd.decode n,None,"nan")
+  else
+    try (CudfAdd.decode n,None,Hashtbl.find tables.reverse_table (n,i))
+    with Not_found -> (
+	warning "Cannot find real version for cudf package %s (= %d)" p i;
+	raise Not_found
+    )
 
 let encode_vpkgname ?arch ?(archs=[]) vpkgname =
   let aux name = function
@@ -147,7 +188,7 @@ let loadl tables ?arch ?(archs=[]) l =
   List.flatten (
     List.map (fun ((name,_) as vpkgname,constr) ->
       List.map (fun encname ->
-        match CudfAdd.cudfop constr with
+        match pefcudf_constr constr with
         |None -> (encname, None)
         |Some(op,v) -> (encname,Some(op,get_cudf_version tables (name,v)))
       ) (encode_vpkgname ?arch ~archs vpkgname)
@@ -158,7 +199,7 @@ let loadlp tables ?arch ?(archs=[]) l =
   List.flatten (
     List.map (fun ((name,_) as vpkgname,constr) ->
       List.map (fun encname ->
-        match CudfAdd.cudfop constr with
+        match pefcudf_constr constr with
         |None -> (encname, None)
         |Some(`Eq,v) -> (encname,Some(`Eq,get_cudf_version tables (name,v)))
         |_ -> assert false
@@ -231,14 +272,4 @@ let load_universe compare l =
   let univ = Cudf.load_universe pkglist in
   Util.Timer.stop timer univ
 
-(** convert a pef constraint into a cudf constraint *)
-let pefvpkg to_cudf vpkgname =
-  let constr n constr =
-    match CudfAdd.cudfop constr with
-    |None -> None
-    |Some(op,v) -> Some(op,snd(to_cudf (n,v)))
-  in
-  match vpkgname with
-  |((n,None),c) -> (CudfAdd.encode n,constr n c)
-  |((n,Some ("any"|"native")),c) -> (CudfAdd.encode n,constr n c)
-  |((n,Some a),c) -> (CudfAdd.encode (n^":"^a),constr n c)
+
